package Kephra::Edit::Format;
$VERSION = '0.23';

use strict;
use Wx qw(
	wxSTC_CMD_NEWLINE wxSTC_CMD_DELETEBACK wxSTC_CMD_LINEEND
	wxSTC_CMD_WORDLEFT wxSTC_CMD_WORDRIGHT
);

# change indention width of selected text
sub _indent_selection {
	my $width = shift || 0;
	my $ep    = Kephra::App::EditPanel::_get();
	$ep->BeginUndoAction;
	$ep->SetLineIndentation( $_, $ep->GetLineIndentation($_) + $width ) 
		for $ep->LineFromPosition($ep->GetSelectionStart)
		 .. $ep->LineFromPosition($ep->GetSelectionEnd);
	$ep->EndUndoAction;
}

sub indent_space { _indent_selection( 1) }
sub dedent_space { _indent_selection(-1) }
sub indent_tab   { _indent_selection( $Kephra::document{current}{tab_size}) }
sub dedent_tab   { _indent_selection(-$Kephra::document{current}{tab_size}) }

#
sub align_indent {
	my $ep = Kephra::App::EditPanel::_get();
	my $firstline = $ep->LineFromPosition( $ep->GetSelectionStart );
	my $align = $ep->GetLineIndentation($firstline);
	$ep->BeginUndoAction();
	$ep->SetLineIndentation($_ ,$align)
		for $firstline + 1 .. $ep->LineFromPosition($ep->GetSelectionEnd);
	$ep->EndUndoAction();
}

# deleting trailing spaces on line ends
sub del_trailing_spaces {
	&Kephra::Edit::_save_positions;
	my $ep = Kephra::App::EditPanel::_get();
	my $text = Kephra::Edit::_select_all_if_none();
	$text =~ s/[ \t]+(\r|\n|\Z)/$1/g;
	$ep->BeginUndoAction;
	$ep->ReplaceSelection($text);
	$ep->EndUndoAction;
	Kephra::Edit::_restore_positions();
}

#
sub join_lines {
 my $ep = Kephra::App::EditPanel::_get();
 my $text = $ep->GetSelectedText();
	$text =~ tr/\r\n//d; # delete end of line marker
	$ep->BeginUndoAction;
	$ep->ReplaceSelection($text);
	$ep->EndUndoAction;
}

sub blockformat{
	return unless Scalar::Util::looks_like_number($_[0]);
	my $width      = (int shift) + 1;
	my $ep         = Kephra::App::EditPanel::_get();
	my ($begin, $end) = $ep->GetSelection;
	my $bline      = $ep->LineFromPosition($begin);
	my $tmp_begin  = $ep->PositionFromLine($bline);
	my $bspace     = ' ' x $ep->GetLineIndentation($bline);
	my $space      = $Kephra::config{editpanel}{auto}{indention} ? $bspace : '';
	chop $bspace;

	$ep->SetSelection($tmp_begin, $end);
	require Text::Wrap;
	$Text::Wrap::columns  = $width;
	$Text::Wrap::unexpand = $Kephra::document{current}{tab_use};
	$Text::Wrap::tabstop  = $Kephra::document{current}{tab_size};

	my $text = $ep->GetSelectedText;
	$text =~ tr/\r\n/ /d;
	$ep->BeginUndoAction();
	$ep->ReplaceSelection( Text::Wrap::fill($bspace, $space, $text) );
	$ep->EndUndoAction();
}

sub blockformat_LLI{
	blockformat( $Kephra::config{editpanel}{indicator}{right_margin}{position} );
}

sub blockformat_custom{
	my $width = Kephra::Dialog::get_text( Kephra::App::Window::_get(),
			$Kephra::localisation{dialog}{edit}{wrap_width_input},
			$Kephra::localisation{dialog}{edit}{wrap_custom_headline}
	);
	blockformat( $width ) if defined $width;
}



# breaking too long lines into smaller one
sub line_break {
	return unless Scalar::Util::looks_like_number($_[0]);
	my $width        = (int shift) + 1;
	my $ep           = Kephra::App::EditPanel::_get();
	my ($begin, $end)= $ep->GetSelection;
	my $tmp_begin    = $ep->LineFromPosition( $ep->PositionFromLine($begin) );

	$ep->SetSelection($tmp_begin, $end);
	require Text::Wrap;
	$Text::Wrap::columns  = $width;
	$Text::Wrap::unexpand = $Kephra::document{current}{tab_use};
	$Text::Wrap::tabstop  = $Kephra::document{current}{tab_size};

	$ep->BeginUndoAction();
	$ep->ReplaceSelection( Text::Wrap::wrap('', '', $ep->GetSelectedText) );
	$ep->EndUndoAction();
}

sub linebreak_custom {
	my $l10n = $Kephra::localisation{dialog}{edit};
	my $width = Kephra::Dialog::get_text( Kephra::App::Window::_get(),
			$l10n->{wrap_width_input}, $l10n->{wrap_custom_headline} );
	line_break( $width ) if defined $width;
}

sub linebreak_LLI {
	line_break( $Kephra::config{editpanel}{indicator}{right_margin}{position} );
}

sub linebreak_window {
	my $app     = Kephra::App::Window::_get();
	my $ep = Kephra::App::EditPanel::_get();
	my ($width) = $app->GetSizeWH();
	my $pos = $ep->GetColumn($ep->PositionFromPointClose(100, 67));
	Kephra::Dialog::msg_box( $app, $pos, '' );
	#line_break($width);
}

1;
